#version 130
#define FARE_PLAN 200
precision highp float;


uniform float pulsLigthPos;
uniform float pulsLightDistance;
uniform bool wireFrameNet;
uniform bool depthMask;
uniform sampler2D texture_color;
uniform float startRad; // WOBBLE
//uniform float glowAlpha;
uniform float glowLumOffset;
uniform float lumFactor;

in vec4 vert_color;
in float vert_tunnelLength;
in vec2 frag_texCoord;
in vec4 fragPerspPos;


const float pulsLightWidth = 2.5f;
const vec4 pulsColorSub = vec4(1.0f, 1.0f, -0.0f, 0.0f);
//const vec4 pulsColorSub = vec4(0.0f, 0.0f, -0.0f, 0.0f);
const vec4 white = vec4(1.0f, 1.0f, 1.0f, 1.0f);

// WOBBLE
const float C_PI    = 3.1415;
const float C_2PI   = 2.0 * C_PI;
const float C_2PI_I = 1.0 / (2.0 * C_PI);
const float C_PI_2  = C_PI / 2.0;

const vec2  frequency = vec2(4.0f, 4.0f);
const vec2  amplitude = vec2(0.06f, 0.02f);


out vec4 fragColor;
out vec4 mbControll;

float distanceToPulseLight()
{
	float fragPos = mod(vert_tunnelLength, pulsLightDistance);
	
	float PL2EndDistance = (pulsLightDistance - pulsLigthPos);
	float Frag2EndDistance = (pulsLightDistance - fragPos);
	
	float correctedPLP;
	if(fragPos < pulsLigthPos && fragPos < pulsLightWidth && pulsLigthPos > (pulsLightDistance - pulsLightWidth))
	{
		correctedPLP = -PL2EndDistance;
	}
	else if(fragPos > pulsLigthPos && fragPos > (pulsLightDistance - pulsLightWidth) && pulsLigthPos < pulsLightWidth)
	{
		correctedPLP = pulsLightDistance + pulsLigthPos;
	}
	else
	{
		correctedPLP = pulsLigthPos;
	}
	
	return abs(correctedPLP - fragPos);	
}

vec4 wobbleTexel()
{
	vec2  perturb;
    float rad;
    vec3  color;

    // Compute a perturbation factor for the x-direction
    rad = (frag_texCoord.s + frag_texCoord.t - 1.0 + startRad) * frequency.x;
	
	 // Wrap to -2.0*PI, 2*PI
    rad = rad * C_2PI_I;
    rad = fract(rad);
    rad = rad * C_2PI;

    // Center in -PI, PI
    if (rad >  C_PI) rad = rad - C_2PI;
    if (rad < -C_PI) rad = rad + C_2PI;

    // Center in -PI/2, PI/2
    if (rad >  C_PI_2) rad =  C_PI - rad;
    if (rad < -C_PI_2) rad = -C_PI - rad;

    perturb.x = (rad - (rad * rad * rad / 6.0)) * amplitude.x;

    // Now compute a perturbation factor for the y-direction
    rad = (frag_texCoord.s - frag_texCoord.t + startRad) * frequency.y;

    // Wrap to -2*PI, 2*PI
    rad = rad * C_2PI_I;
    rad = fract(rad);
    rad = rad * C_2PI;

    // Center in -PI, PI
    if (rad >  C_PI) rad = rad - C_2PI;
    if (rad < -C_PI) rad = rad + C_2PI;

    // Center in -PI/2, PI/2
    if (rad >  C_PI_2) rad =  C_PI - rad;
    if (rad < -C_PI_2) rad = -C_PI - rad;

    perturb.y = (rad - (rad * rad * rad / 6.0)) * amplitude.y;

    return texture2D(texture_color, perturb + frag_texCoord.st);
   
    //return vec4(1.0f, 1.0f, -0.0f, 0.0f);
}

void main()
{
	//fragColor = vec4(1.0f, 0.0f, 0.0f, 1.0f);
	if(!depthMask)
	{
	
		if(wireFrameNet)
		{
			fragColor = vert_color * 1.2f;
			//fragColor.a = 0.6f;
		}
		else
		{
			//fragColor = texture2D(texture_color, frag_texCoord.st);
			fragColor = wobbleTexel();
			//fragColor.a = 1.0f - (length(fragColor.rgb) / 3.0f); // desto weiser desto transparenter
			//fragColor.a = fragColor.g; // wehr zwar durchs blending program "GL_ONE" fr den eigentlichen tunnel nicht notwendig, aber das power light schaut damit geiler aus...
			fragColor = fragColor * 0.3f; //0.3f; // abdunkeln
			//fragColor.g += 0.05f;
			//fragColor.a = fragColor.a/2.0f;
		}
		
		
		float absDistFragPL = distanceToPulseLight();
		float invDistProzent = 0;
		if(absDistFragPL < pulsLightWidth)
		{
			invDistProzent = (pulsLightWidth - absDistFragPL) / pulsLightWidth;
			fragColor = fragColor + white * pow(invDistProzent, 1.5);
			fragColor = min(fragColor, 1.0f);
			fragColor = fragColor - pulsColorSub * pow(invDistProzent, 3);
			//fragColor.a = fragColor.a + invDistProzent * 1.5f;
			//fragColor = pulsColor;
			fragColor *= 3.0f; // *2.0f sieht schon seh ordentlich aus, aber damit man die adaptiveLum anpassung beser sieht knall ichs noch rauf
		}
		// damit ich frs glown eine tifeninformation habe, misbrauche ich den alpha kanal
		
	//if(!depthMask) {
		
			float lumOffset = 0.1f;
			float blendOutOffset = 0.12f; // 0.1f !!!!
			float blendOutHardnes = 4.0f;
			float blendInMultiplayer = 10.0f;
			float finalColorMultiplier = lumFactor; // 1.8f; // 2.5f !!!!!
			
			float distance = (fragPerspPos.z/FARE_PLAN); // /2.0f);
			
			float distanceWight;
			if(distance < blendOutOffset)
				distanceWight = 1.0f + (blendOutOffset - distance) * blendInMultiplayer;
			else
				distanceWight = 1.0f - pow(distance - blendOutOffset, 1.0f / blendOutHardnes);
			
			distanceWight += lumOffset;
			
			//distanceWight = min(distanceWight, 1.0f);
			//distanceWight = max(distanceWight, 0.01f);
			
			fragColor = (fragColor * distanceWight) * finalColorMultiplier + glowLumOffset; // vec4(0.0f, 0.0f, 0.0f, 0.0f); // *5.0f is ohne tonemaping super! 10.0f is mit tonmapping ganz anembar
			//fragColor = vec4(distanceWight, distanceWight, distanceWight, 1.0f);
			
			//if(distance > 0.1f)
			//	fragColor = vec4(1.0f, 1.0f, 1.0f, 2.0f);
			
			//fragColor = vec4(distanceWight, distanceWight, distanceWight, distanceWight);
			
			/*
			fragColor.a = pow(distanceWight, 8.0f) / 8.0f;
			fragColor.a = max(fragColor.a, 0.4f); // 0.3f
			fragColor.a = fragColor.a + invDistProzent; // damit das puls licht das durch den tunnel leuft auch wirklich glowed
			fragColor.a = 1.0f;
			// nur dort wo auch wirklich grn
			/*if(fragColor.g < 0.1)
			{
				fragColor.a = 0.0f;
			}*/
			
			// keine anung warum openGL das braucht:
			gl_FragDepth = gl_FragCoord.z;
			mbControll = vec4(0.0f, 0.0f, 0.0f, 0.0f);	
	}
	else
	{
		gl_FragDepth = gl_FragCoord.z;
		/*
		// depth value nur dort schrieben wo auch wirklich farbe ist
		if(length(fragColor.rgb) > 0.05f)
		{
			gl_FragDepth = gl_FragCoord.z;
		}
		else
		{
			gl_FragDepth = 1.0f; // ganz an die far plan schreiben, sollte also von allem andern berschrieben werden
		}
		*/
		fragColor = vec4(0.0f, 0.0f, 0.0f, 0.0f);
	}
}